# Python bytecode 2.7 (decompiled from Python 2.7)
# Embedded file name: e:\jenkins\workspace\client_SERENITY\branches\release\SERENITY\carbon\common\script\util\timerstuff.py
import log
import sys
import blue
import uthread
import bluepy
import weakref
import traceback
import functools
import const
import stackless
ClockThis = sys.ClockThis
ClockThisWithoutTheStars = sys.ClockThisWithoutTheStars

def TimeThis(timer):

    def Helper(func):

        @functools.wraps(func)
        def Wrapper(*args, **kwds):
            t = bluepy.PushTimer(timer)
            try:
                return func(*args, **kwds)
            finally:
                bluepy.PopTimer(t)

        return Wrapper

    return Helper


class SimpleTimingContext(object):

    def __enter__(self):
        c = stackless.getcurrent()
        self.t = (c.GetWallclockTime(), c.GetRunTime())

    def __exit__(self, exc_type, exc_value, exc_tb):
        a, b = self.t
        c = stackless.getcurrent()
        self.t = (c.GetWallclockTime() - a, c.GetRunTime() - b)

    def GetWallclockTime(self):
        return self.t[0]

    def GetRunTime(self):
        return self.t[1]


class TracebackTimingContext(SimpleTimingContext):

    def __init__(self, runtime=1.0, wallclock=None):
        self.wallclock, self.runtime = wallclock, runtime

    def __exit__(self, exc_type, exc_val, exc_tb):
        SimpleTimingContext.__exit__(self, exc_type, exc_val, exc_tb)
        if self.wallclock is not None and self.GetWallclockTime() > self.wallclock or self.runtime is not None and self.GetRunTime() > self.runtime:
            log.LogTraceback('TracebackTimingContext exceeded: wallclock=%s, runtime=%s' % (self.GetWallclockTime(), self.GetRunTime()))
        return


def TracebackTimingFunction(runtime=1.0, wallclock=None):

    def decorator(f):

        @functools.wraps(f)
        def wrapper(*args, **kwds):
            with TracebackTimingContext(runtime=runtime, wallclock=wallclock):
                return f(*args, **kwds)

        return wrapper

    return decorator


class TimerObject:
    __guid__ = 'base.TimerObject'

    def __init__(self, useSimTime=False):
        self.abort = 0
        self.id = 0
        self.aborts = {}
        if useSimTime:
            self.GetTime = blue.os.GetSimTime
            self.Sleep = blue.pyos.synchro.SleepSim
        else:
            self.GetTime = blue.os.GetWallclockTime
            self.Sleep = blue.pyos.synchro.SleepWallclock

    def __del__(self):
        self.abort = 1

    def DelayedCall(self, func, time, *args, **kw):
        self.id += 1
        callID = self.id
        if time - self.GetTime() > const.DAY:
            raise RuntimeError('Cannot sleep for more than 24 hours')
        uthread.new(self.DelayedCall_thread, callID, func, time, args, kw)
        return callID

    def DelayedCall_thread(self, callID, func, time, args, kw):
        delay = (time - self.GetTime()) / 10000
        if delay > 0:
            self.Sleep(delay)
        if callID in self.aborts:
            del self.aborts[callID]
        elif not self.abort:
            try:
                func(*args, **kw)
            except:
                log.LogException('Error in delayed call')

    def KillDelayedCall(self, id):
        self.aborts[id] = None
        return

    def KillAllDelayedCalls(self):
        self.abort = 1


def OnTimerResync(old, new):
    diff = new - old
    for timer in AutoTimer.autoTimers.keys():
        timer.wakeupAt += diff / 10000


class AutoTimer(object):
    __guid__ = 'base.AutoTimer'
    autoTimers = weakref.WeakKeyDictionary()
    blue.pyos.synchro.timesyncs.append(OnTimerResync)

    def __init__(self, interval, method, *args, **kw):
        if interval <= 0:
            raise RuntimeError('Invalid interval for AutoTimer')
        self.run = 1
        self.interval = interval
        self.method, self.args, self.kw = method, args, kw
        caller_tb = traceback.extract_stack(limit=2)[0:1]
        caller = traceback.format_list(caller_tb)[0].replace('\n', '')
        methrepr = getattr(method, '__name__', '(no __name__)').replace('<', '&lt;').replace('>', '&gt;')
        ctx = 'AutoTimer::(%s) on %s' % (caller, methrepr)
        self.wakeupAt = blue.os.GetWallclockTime() / 10000 + self.interval
        AutoTimer.autoTimers[self] = None
        uthread.pool(ctx, AutoTimer.Loop, weakref.ref(self))
        return

    def KillTimer(self):
        self.run = 0

    def Reset(self, newInterval=None):
        if newInterval is not None:
            if newInterval <= 0:
                raise RuntimeError('Invalid interval for AutoTimer')
            self.interval = newInterval
        self.wakeupAt = blue.os.GetWallclockTime() / 10000 + self.interval
        return

    def Loop(weakSelf):
        self = weakSelf()
        if not self or not self.run:
            return
        nap = self.interval
        del self
        while True:
            blue.pyos.synchro.SleepWallclock(nap)
            self = weakSelf()
            if not self or not self.run:
                return
            now = blue.os.GetWallclockTime() / 10000
            if now < self.wakeupAt:
                nap = self.wakeupAt - now
            else:
                self.method(*self.args, **self.kw)
                n = (now - self.wakeupAt) / self.interval
                self.wakeupAt += (n + 1) * self.interval
                nap = self.wakeupAt - now
            del self

    Loop = staticmethod(Loop)


def AutoTimerTest():
    print 'fewbar'


class Stopwatch:

    def __init__(self):
        self.started = blue.os.GetCycles()[0]

    def __str__(self):
        return '%.3f' % ((blue.os.GetCycles()[0] - self.started) / float(blue.os.GetCycles()[1]))

    def Reset(self):
        self.started = blue.os.GetCycles()[0]


exports = {'base.AutoTimerTest': AutoTimerTest,
 'timerstuff.ClockThis': ClockThis,
 'timerstuff.ClockThisWithoutTheStars': ClockThisWithoutTheStars,
 'util.Stopwatch': Stopwatch,
 'timerstuff.TimeThis': TimeThis,
 'timerstuff.SimpleTimingContext': SimpleTimingContext,
 'timerstuff.TracebackTimingContext': TracebackTimingContext,
 'timerstuff.TracebackTimingFunction': TracebackTimingFunction}